# exercise 11.1.1
import importlib_resources
import numpy as np
from matplotlib.pyplot import figure, show
from scipy.io import loadmat
from sklearn.mixture import GaussianMixture

from dtuimldmtools import clusterplot

filename = importlib_resources.files("dtuimldmtools.data").joinpath("synth2.mat")

# Load Matlab data file and extract variables of interest
mat_data = loadmat(filename)
X = mat_data["X"]
y = mat_data["y"].squeeze()
attributeNames = [name[0] for name in mat_data["attributeNames"].squeeze()]
classNames = [name[0][0] for name in mat_data["classNames"]]
# X_old = X
# X = np.hstack([X,X])
N, M = X.shape
C = len(classNames)
# Number of clusters
K = 4
cov_type = "full"  # e.g. 'full' or 'diag'

# define the initialization procedure (initial value of means)
initialization_method = "random"  #  'random' or 'kmeans'
# random signifies random initiation, kmeans means we run a K-means and use the
# result as the starting point. K-means might converge faster/better than
# random, but might also cause the algorithm to be stuck in a poor local minimum

# type of covariance, you can try out 'diag' as well
reps = 1
# number of fits with different initalizations, best result will be kept
# Fit Gaussian mixture model
gmm = GaussianMixture(
    n_components=K,
    covariance_type=cov_type,
    n_init=reps,
    tol=1e-6,
    reg_covar=1e-6,
    init_params=initialization_method,
).fit(X)
cls = gmm.predict(X)
# extract cluster labels
cds = gmm.means_
# extract cluster centroids (means of gaussians)
covs = gmm.covariances_
# extract cluster shapes (covariances of gaussians)
if cov_type.lower() == "diag":
    new_covs = np.zeros([K, M, M])

    count = 0
    for elem in covs:
        temp_m = np.zeros([M, M])
        new_covs[count] = np.diag(elem)
        count += 1

    covs = new_covs

# Plot results:
figure(figsize=(14, 9))
clusterplot(X, clusterid=cls, centroids=cds, y=y, covars=covs)
show()

## In case the number of features != 2, then a subset of features most be plotted instead.
# figure(figsize=(14,9))
# idx = [0,1] # feature index, choose two features to use as x and y axis in the plot
# clusterplot(X[:,idx], clusterid=cls, centroids=cds[:,idx], y=y, covars=covs[:,idx,:][:,:,idx])
# show()

print("Ran Exercise 11.1.1")
